@@ -27,16 +27,14 @@ WikiJS options:
   --wiki-title=<title>              The title to use for the wiki website.
   --wiki-git-branch=<wiki_branch>   The name of the branch that the wiki content should be loaded from. [default: main]
   --wiki-git-no-verify              Do not verify the ssl certificate when cloning the wikijs wiki.
+  --wiki-navigation=<type>          The type of wiki navigation to confgigure, either 'TREE' or 'NONE'. [default: TREE]
 
 Docker options:
   -t --tag=<tag>                    The docker tag to use for the generated image. This should exclude the registry portion. [default: sci-oer/custom:latest]
   -b --base=<base>                  The base image to use [default: marshallasch/java-resource:latest]
-  --registry=<url>                  The url of a docker registry to push the image to
   --no-pull                         Don't pull the base image first
-  --no-push                         Don't push the image to a remote registry.
-  --save-tar=<file name>            The name of the tar archive to export the image as.
-                                    If running in docker it will get put in the `/output` directory,
-                                    if the builder is being run on the host then it will be palced in the `./output` directory.
+  --push                            Push the image to the DockerHub registry.
+  --multi-arch                      Build the docker image for amd64 and arm64. [default: False]
 
 Other interface options:
   -h --help                         Show this help message.
@@ -57,6 +55,7 @@ import platform
 import requests
 import json
 import copy
+import subprocess
 
 from docopt import docopt
 import colorlog
@@ -109,19 +108,9 @@ def ask_interactive(opts):
         default="no" if input["no_pull"] else "yes",
     )
 
-    input["no_push"] = not yesno(
-        "Do you want to push the generated image to a docker registry?",
-        default="no" if input["no_push"] else "yes",
-    )
-    if not input["no_push"]:
-        input["registry"] = prompt(
-            "Enter the registry that the generated should be pushed too (leave blank for dockerhub):",
-            default=input["registry"],
-        )
-
-    input["save_tar"] = prompt(
-        "Enter name of the tar file of the generated image to save (leave blank if you do not want to save it):",
-        default=input["save_tar"],
+    input["push"] = yesno(
+        "Do you want to push the generated image to dockerhub?",
+        default="yes" if input["push"] else "no",
     )
 
     print("")
@@ -219,6 +208,14 @@ def start_container(client, volume, image, keyFile, **kwargs):
     return container
 
 
+def change_key_permissions(container, keyFile, **kwargs):
+    _LOGGER.info(
+        "Copying the SSH key within the container to set the correct ownership"
+    )
+    container.exec_run(f'sudo cp "{keyFile}" "{keyFile}.CONTAINER"')
+    container.exec_run(f'sudo chown 1000:1000 "{keyFile}.CONTAINER"')
+
+
 def stop_container(container, **kwargs):
     _LOGGER.info("stopping docker container...")
     container.stop()
@@ -294,7 +291,9 @@ def cleanup_build(dir):
     dir.cleanup()
 
 
-def build_image(client, dir, tag="sci-oer:custom", base=None, **kwargs):
+def build_multi_arch(
+    client, dir, tag="sci-oer:custom", base=None, push=False, **kwargs
+):
 
     args = {
         "BASE_IMAGE": base,
@@ -302,27 +301,37 @@ def build_image(client, dir, tag="sci-oer:custom", base=None, **kwargs):
     }
 
     _LOGGER.info(f"Building custom image with name `{tag}`...")
+
+    # platforms = 'linux/amd64,linux/arm64,linux/amd64/v2,linux/arm/v7,linux/arm/v6,linux/386'
+    platforms = "linux/amd64,linux/arm64"
+
+    buildArgs = ""
+    for k, v in args.items():
+        buildArgs = f"{buildArgs}--build-arg {k}={v} "
+
+    push = "" if not push else "--push"
+
+    cmd = f"docker buildx build --progress=plain --platform {platforms} --tag {tag} {push} {buildArgs} {dir}"
+    _LOGGER.debug(f"build command: `{cmd}`")
+    subprocess.run(cmd, shell=True, check=True)
+    _LOGGER.info("Done building custom image.")
+
+
+def build_single_arch(client, dir, tag="sci-oer:custom", base=None, **kwargs):
+
+    args = {
+        "BASE_IMAGE": base,
+        "BUILD_DATE": datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ"),
+    }
+
+    _LOGGER.info(f"Building custom image with name `{tag}`...")
+
     image, logs = client.images.build(path=dir, tag=tag, buildargs=args)
     _LOGGER.info("Done building custom image.")
 
     return image
 
 
-def save_image(image, fileName, directory, **kwargs):
-
-    os.makedirs(directory, exist_ok=True)
-
-    tarFile = os.path.join(directory, fileName)
-
-    _LOGGER.info(f'Saving image to "{tarFile}"...')
-    f = open(tarFile, "wb")
-    for chunk in image.save():
-        f.write(chunk)
-    f.close()
-
-    _LOGGER.info("Done saving image to tar file.")
-
-
 def extract_db(container, dir, **kwargs):
     _LOGGER.info("extracting wikijs database...")
     f = open(os.path.join(dir, "database.sqlite.tar"), "wb")
@@ -433,6 +442,23 @@ def set_wiki_title(host, title, **kwargs):
     api_call(host, query, variables={"title": title}, **kwargs)
 
 
+def set_wiki_navigation_mode(host, mode, **kwargs):
+
+    if mode not in ["NONE", "TREE"]:
+        _LOGGER.warning(f"Invalid navigation mode '{mode}'")
+        return
+
+    query = """mutation Navigation ($mode: NavigationMode!){
+        navigation {
+            updateConfig (mode: $mode ) {
+                responseResult { succeeded, errorCode, slug, message }
+            }
+        }
+    }"""
+
+    api_call(host, query, variables={"mode": mode}, **kwargs)
+
+
 def load_ssh_key(keyFile):
 
     env_key_file = os.getenv(SSH_KEY_FILE_ENV)
@@ -495,7 +521,7 @@ def configure_wiki_repo(host, enabled, repo, **kwargs):
                     },
                     {
                         "key": "sshPrivateKeyPath",
-                        "value": f'{{"v": "{repo.auth.ssh_file}"}}',
+                        "value": f'{{"v": "{repo.auth.ssh_file}.CONTAINER"}}',
                     },
                     {
                         "key": "sshPrivateKeyContent",
@@ -604,13 +630,20 @@ def run(opts, **kwargs):
 
     # Checking incompatible arguments
 
+    opts["wiki_navigation"] = opts["wiki_navigation"].strip().upper()
+    if opts["wiki_navigation"] not in ["TREE", "NONE"]:
+        _LOGGER.error(
+            f"'{opts['wiki_navigation']}' is not a valid navigation option must be one of ['TREE', 'NONE']."
+        )
+        sys.exit("Incompatible arguments")
+
     if opts["lectures_repo"] is not None and opts["lectures_directory"] is not None:
         _LOGGER.error(
             "Cannot specify both `--lectures-repo` and `--lectures-directory`, only one can be used at a time."
         )
         sys.exit("Incompatible arguments")
 
-    sshKeyFile = load_ssh_key(opts["key_file"])
+    sshKeyFile = load_ssh_key(os.path.expanduser(opts["key_file"]))
     if sshKeyFile != "":
         sshKeyFile = os.path.realpath(sshKeyFile)
 
@@ -717,11 +750,14 @@ def run(opts, **kwargs):
         )
         wikiRepo.auth = gitAuthentication
 
+        if sshKeyFile:
+            change_key_permissions(container, sshKeyFile)
         set_wiki_contents(host, wikiRepo, port=port, keep_git=opts["keep_git"])
     else:
         _LOGGER.info("wiki content repository has not been set. skipping...")
 
     set_wiki_title(host, opts["wiki_title"], port=port)
+    set_wiki_navigation_mode(host, opts["wiki_navigation"], port=port)
     dissable_api(host, port=port)
 
     stop_container(container)
@@ -733,7 +769,15 @@ def run(opts, **kwargs):
     delete_container(container)
     delete_volume(volume)
 
-    image = build_image(client, dir.name, tag=opts["tag"], base=opts["base"])
+    if opts["multi_arch"]:
+        _LOGGER.info("Starting multi platform build")
+        build_multi_arch(
+            client, dir.name, tag=opts["tag"], base=opts["base"], push=opts["push"]
+        )
+    else:
+        _LOGGER.info("Starting single platform build")
+        build_single_arch(client, dir.name, tag=opts["tag"], base=opts["base"])
+
     cleanup_build(dir)
 
     if containerized:
@@ -742,12 +786,6 @@ def run(opts, **kwargs):
 
     _LOGGER.info("Done.")
 
-    if opts["save_tar"] is not None:
-        save_image(image, opts["save_tar"], "/output" if containerized else "./output")
-
-    if not opts["no_push"]:
-        push_image(client, opts["registry"], image)
-
 
 def main():
     args = docopt(__doc__)
@@ -798,6 +836,14 @@ def main():
         print(f"scioer-builder: {__version__}")
         sys.exit(0)
 
+    try:
+        docker.from_env()
+    except:
+        _LOGGER.error(
+            "failed to connect to docker, check that Docker is running on the host."
+        )
+        sys.exit("Docker is not running")
+
     opts = _make_opts(args)
     _LOGGER.info(opts)
 
